#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "phoebe_error_handling.h"
#include "phoebe_global.h"
#include "phoebe_gui_accessories.h"
#include "phoebe_gui_support.h"
#include "phoebe_keywords.h"

void declare_parameter (char *qualifier, GtkWidget *parent, char *widget, char *keyword, char *description)
	{
	/* This function adds an empty record to the PHOEBE_parameter struct (defi- */
	/* ned in phoebe_global.h dynamically and fills it in with values given as  */
	/* arguments to this function. It should be called *only* by declare_all_   */
	/* _parameters () function. The strdup () function allocates memory for the */
	/* string automatically by calling malloc (). All allocations are freed     */
	/* upon PHOEBE exit.                                                        */

	int i = PHOEBE_parameter.recno;

	PHOEBE_parameter.recno++;
	PHOEBE_parameter.widget      = realloc (PHOEBE_parameter.widget, PHOEBE_parameter.recno * sizeof (GtkWidget *));
	PHOEBE_parameter.qualifier   = realloc (PHOEBE_parameter.qualifier, PHOEBE_parameter.recno * sizeof (char *));
	PHOEBE_parameter.keyword     = realloc (PHOEBE_parameter.keyword, PHOEBE_parameter.recno * sizeof (char *));
	PHOEBE_parameter.description = realloc (PHOEBE_parameter.description, PHOEBE_parameter.recno * sizeof (char *));

	PHOEBE_parameter.widget[i]      = lookup_widget (parent, widget);
	PHOEBE_parameter.qualifier[i]   = strdup (qualifier);
	PHOEBE_parameter.keyword[i]     = strdup (keyword);
	PHOEBE_parameter.description[i] = strdup (description);
	}

void declare_all_parameters ()
	{
	/* This (and only this) function holds all name associations with the GUI;  */
	/* if a name within a GUI is changed, you should fix the association here.  */

	/* **********************   Data tab parameters   ************************* */

	declare_parameter ("phoebe_name_value",          PHOEBE, "data_star_name_entry",                    "NAME",          "Star name");
	declare_parameter ("phoebe_model_value",         PHOEBE, "data_model_list_entry",                   "MODEL",         "Model (applied constraints)");
	declare_parameter ("phoebe_lcno_value",          PHOEBE, "data_lc_no_value",                        "LCNO",          "Number of experimental light curves");
	declare_parameter ("phoebe_rvno_value",          PHOEBE, "data_rv_no_value",                        "RVNO",          "Number of experimental radial velocity curves");
	declare_parameter ("phoebe_spno_value",          PHOEBE, "data_sp_no_value",                        "SPNO",          "Number of experimental spectra");
	declare_parameter ("phoebe_mnorm_value",         PHOEBE, "data_mnorm_value",                        "MNORM",         "Flux-normalizing magnitude");
	declare_parameter ("phoebe_binning_switch",      PHOEBE, "data_binning_switch",                     "BINSWITCH",     "Data binning");
	declare_parameter ("phoebe_binno_value",         PHOEBE, "data_binning_value",                      "BINNO",         "Number of bins");
	declare_parameter ("phoebe_reddening_switch",    PHOEBE, "data_lc_options_remove_reddening_switch", "REDSWITCH",     "Interstellar extinction (reddening)");
	declare_parameter ("phoebe_reddeningr_value",    PHOEBE, "data_lc_options_r_value",                 "REDFACTOR",     "Interstellar extinction coefficient");
	declare_parameter ("phoebe_reddeninge_value",    PHOEBE, "data_lc_options_color_excess_value",      "REDEXCESS",     "Interstellar extinction color excess value");
	declare_parameter ("phoebe_rv1proximity_switch", PHOEBE, "data_rv_options_icor1_switch",            "RV1PROXSWITCH", "Proximity effects for primary star");
	declare_parameter ("phoebe_rv2proximity_switch", PHOEBE, "data_rv_options_icor2_switch",            "RV2PROXSWITCH", "Proximity effects for secondary star");
	declare_parameter ("phoebe_lc_filename",         PHOEBE, "data_lc_info_list_column_2",              "LCFILENAME",    "Experimental LC data filename");
	declare_parameter ("phoebe_lc_sigma",            PHOEBE, "data_lc_info_list_column_3",              "LCSIGMA",       "Experimental LC data standard deviation");
	declare_parameter ("phoebe_lc_filter",           PHOEBE, "data_lc_info_list_column_4",              "LCFILTER",      "Experimental LC data filter");
	declare_parameter ("phoebe_lc_column1",          PHOEBE, "data_lc_info_list_column_5",              "LCCOLUMN1",     "Experimental LC data column 1");
	declare_parameter ("phoebe_lc_column2",          PHOEBE, "data_lc_info_list_column_6",              "LCCOLUMN2",     "Experimental LC data column 2");
	declare_parameter ("phoebe_lc_column3",          PHOEBE, "data_lc_info_list_column_7",              "LCCOLUMN3",     "Experimental LC data column 3");
	declare_parameter ("phoebe_lc_weighting",        PHOEBE, "data_lc_info_list_column_8",              "LCWEIGHTING",   "Experimental LC data weighting");
	declare_parameter ("phoebe_rv_filename",         PHOEBE, "data_rv_info_list_column_2",              "RVFILENAME",    "Experimental RV data filename");
	declare_parameter ("phoebe_rv_sigma",            PHOEBE, "data_rv_info_list_column_3",              "RVSIGMA",       "Experimental RV data standard deviation");
	declare_parameter ("phoebe_rv_filter",           PHOEBE, "data_rv_info_list_column_4",              "RVFILTER",      "Experimental RV data filter");
	declare_parameter ("phoebe_rv_column1",          PHOEBE, "data_rv_info_list_column_5",              "RVCOLUMN1",     "Experimental RV data column 1");
	declare_parameter ("phoebe_rv_column2",          PHOEBE, "data_rv_info_list_column_6",              "RVCOLUMN2",     "Experimental RV data column 2");
	declare_parameter ("phoebe_rv_column3",          PHOEBE, "data_rv_info_list_column_7",              "RVCOLUMN3",     "Experimental RV data column 3");
	declare_parameter ("phoebe_sp_filename",         PHOEBE, "data_sp_info_list_column_2",              "SPFILENAME",    "Experimental spectra data filename");
	declare_parameter ("phoebe_sp_sigma",            PHOEBE, "data_sp_info_list_column_3",              "SPSIGMA",       "Experimental spectra data standard deviation");
	declare_parameter ("phoebe_sp_filter",           PHOEBE, "data_sp_info_list_column_4",              "SPFILTER",      "Experimental spectra data filter");
	declare_parameter ("phoebe_sp_column1",          PHOEBE, "data_sp_info_list_column_5",              "SPCOLUMN1",     "Experimental spectra data column 1");
	declare_parameter ("phoebe_sp_column2",          PHOEBE, "data_sp_info_list_column_6",              "SPCOLUMN2",     "Experimental spectra data column 2");
	declare_parameter ("phoebe_sp_column3",          PHOEBE, "data_sp_info_list_column_7",              "SPCOLUMN3",     "Experimental spectra data column 3");

	/* *********************   System tab parameters   ************************ */

	declare_parameter ("phoebe_hjd0_value",          PHOEBE, "system_hjd0_value", "HJD0VAL", "Heliocentric Julian date ephemeris origin");
	declare_parameter ("phoebe_hjd0_adjust",         PHOEBE, "system_hjd0_adjust", "HJD0ADJ", "");
	declare_parameter ("phoebe_hjd0_step",           PHOEBE, "system_hjd0_step_value", "HJD0STEP", "");
	declare_parameter ("phoebe_hjd0_min",            PHOEBE, "system_hjd0_min_value", "HJD0MIN", "");
	declare_parameter ("phoebe_hjd0_max",            PHOEBE, "system_hjd0_max_value", "HJD0MAX", "");
	declare_parameter ("phoebe_period_value",        PHOEBE, "system_period_value", "PERIODVAL", "Orbital period in days");
	declare_parameter ("phoebe_period_adjust",       PHOEBE, "system_period_adjust", "PERIODADJ", "");
	declare_parameter ("phoebe_period_step",         PHOEBE, "system_period_step_value", "PERIODSTEP", "");
	declare_parameter ("phoebe_period_min",          PHOEBE, "system_period_min_value", "PERIODMIN", "");
	declare_parameter ("phoebe_period_max",          PHOEBE, "system_period_max_value", "PERIODMAX", "");
	declare_parameter ("phoebe_dpdt_value",          PHOEBE, "system_dpdt_value", "DPDTVAL", "First time derivative of period (days per day)");
	declare_parameter ("phoebe_dpdt_adjust",         PHOEBE, "system_dpdt_adjust", "DPDTADJ", "");
	declare_parameter ("phoebe_dpdt_step",           PHOEBE, "system_dpdt_step_value", "DPDTSTEP", "");
	declare_parameter ("phoebe_dpdt_min",            PHOEBE, "system_dpdt_min_value", "DPDTMIN", "");
	declare_parameter ("phoebe_dpdt_max",            PHOEBE, "system_dpdt_max_value", "DPDTMAX", "");
	declare_parameter ("phoebe_pshift_value",        PHOEBE, "system_pshift_value", "PSHIFTVAL", "Phase shift");
	declare_parameter ("phoebe_pshift_adjust",       PHOEBE, "system_pshift_adjust", "PSHIFTADJ", "");
	declare_parameter ("phoebe_pshift_step",         PHOEBE, "system_pshift_step_value", "PSHIFTSTEP", "");
	declare_parameter ("phoebe_pshift_min",          PHOEBE, "system_pshift_min_value", "PSHIFTMIN", "");
	declare_parameter ("phoebe_pshift_max",          PHOEBE, "system_pshift_max_value", "PSHIFTMAX", "");
	declare_parameter ("phoebe_sma_value",           PHOEBE, "system_sma_value", "SMAVAL", "Semi-major axis in solar radii");
	declare_parameter ("phoebe_sma_adjust",          PHOEBE, "system_sma_adjust", "SMAADJ", "");
	declare_parameter ("phoebe_sma_step",            PHOEBE, "system_sma_step_value", "SMASTEP", "");
	declare_parameter ("phoebe_sma_min",             PHOEBE, "system_sma_min_value", "SMAMIN", "");
	declare_parameter ("phoebe_sma_max",             PHOEBE, "system_sma_max_value", "SMAMAX", "");
	declare_parameter ("phoebe_rm_value",            PHOEBE, "system_rm_value", "RMVAL", "Mass ratio (secondary over primary");
	declare_parameter ("phoebe_rm_adjust",           PHOEBE, "system_rm_adjust", "RMADJ", "");
	declare_parameter ("phoebe_rm_step",             PHOEBE, "system_rm_step_value", "RMSTEP", "");
	declare_parameter ("phoebe_rm_min",              PHOEBE, "system_rm_min_value", "RMMIN", "");
	declare_parameter ("phoebe_rm_max",              PHOEBE, "system_rm_max_value", "RMMAX", "");
	declare_parameter ("phoebe_incl_value",          PHOEBE, "system_incl_value", "INCLVAL", "Inclination in degrees");
	declare_parameter ("phoebe_incl_adjust",         PHOEBE, "system_incl_adjust", "INCLADJ", "");
	declare_parameter ("phoebe_incl_step",           PHOEBE, "system_incl_step_value", "INCLSTEP", "");
	declare_parameter ("phoebe_incl_min",            PHOEBE, "system_incl_min_value", "INCLMIN", "");
	declare_parameter ("phoebe_incl_max",            PHOEBE, "system_incl_max_value", "INCLMAX", "");
	declare_parameter ("phoebe_vga_value",           PHOEBE, "system_vga_value", "VGAVAL", "Center-of-mass velocity in km/s");
	declare_parameter ("phoebe_vga_adjust",          PHOEBE, "system_vga_adjust", "VGAADJ", "");
	declare_parameter ("phoebe_vga_step",            PHOEBE, "system_vga_step_value", "VGASTEP", "");
	declare_parameter ("phoebe_vga_min",             PHOEBE, "system_vga_min_value", "VGAMIN", "");
	declare_parameter ("phoebe_vga_max",             PHOEBE, "system_vga_max_value", "VGAMAX", "");

	/* *******************   Component tab parameters   *********************** */

	declare_parameter ("phoebe_tavh_value",          PHOEBE, "component_tavh_value", "TAVHVAL", "Primary star effective temperature in Kelvin");
	declare_parameter ("phoebe_tavh_adjust",         PHOEBE, "component_tavh_adjust", "TAVHADJ", "");
	declare_parameter ("phoebe_tavh_step",           PHOEBE, "component_tavh_step_value", "TAVHSTEP", "");
	declare_parameter ("phoebe_tavh_min",            PHOEBE, "component_tavh_min_value", "TAVHMIN", "");
	declare_parameter ("phoebe_tavh_max",            PHOEBE, "component_tavh_max_value", "TAVHMAX", "");
	declare_parameter ("phoebe_tavc_value",          PHOEBE, "component_tavc_value", "TAVCVAL", "Secondary star effective temperature in Kelvin");
	declare_parameter ("phoebe_tavc_adjust",         PHOEBE, "component_tavc_adjust", "TAVCADJ", "");
	declare_parameter ("phoebe_tavc_step",           PHOEBE, "component_tavc_step_value", "TAVCSTEP", "");
	declare_parameter ("phoebe_tavc_min",            PHOEBE, "component_tavc_min_value", "TAVCMIN", "");
	declare_parameter ("phoebe_tavc_max",            PHOEBE, "component_tavc_max_value", "TAVCMAX", "");
	declare_parameter ("phoebe_phsv_value",          PHOEBE, "component_phsv_value", "PHSVVAL", "Primary star surface potential");
	declare_parameter ("phoebe_phsv_adjust",         PHOEBE, "component_phsv_adjust", "PHSVADJ", "");
	declare_parameter ("phoebe_phsv_step",           PHOEBE, "component_phsv_step_value", "PHSVSTEP", "");
	declare_parameter ("phoebe_phsv_min",            PHOEBE, "component_phsv_min_value", "PHSVMIN", "");
	declare_parameter ("phoebe_phsv_max",            PHOEBE, "component_phsv_max_value", "PHSVMAX", "");
	declare_parameter ("phoebe_pcsv_value",          PHOEBE, "component_pcsv_value", "PCSVVAL", "Secondary star surface potential");
	declare_parameter ("phoebe_pcsv_adjust",         PHOEBE, "component_pcsv_adjust", "PCSVADJ", "");
	declare_parameter ("phoebe_pcsv_step",           PHOEBE, "component_pcsv_step_value", "PCSVSTEP", "");
	declare_parameter ("phoebe_pcsv_min",            PHOEBE, "component_pcsv_min_value", "PCSVMIN", "");
	declare_parameter ("phoebe_pcsv_max",            PHOEBE, "component_pcsv_max_value", "PCSVMAX", "");
	declare_parameter ("phoebe_logg1_value",         PHOEBE, "component_logg1_value", "LOGG1VAL", "Primary star gravity acceleration [cgs]");
	declare_parameter ("phoebe_logg1_adjust",        PHOEBE, "component_logg1_adjust", "LOGG1ADJ", "");
	declare_parameter ("phoebe_logg1_step",          PHOEBE, "component_logg1_step_value", "LOGG1STEP", "");
	declare_parameter ("phoebe_logg1_min",           PHOEBE, "component_logg1_min_value", "LOGG1MIN", "");
	declare_parameter ("phoebe_logg1_max",           PHOEBE, "component_logg1_max_value", "LOGG1MAX", "");
	declare_parameter ("phoebe_logg2_value",         PHOEBE, "component_logg2_value", "LOGG2VAL", "Secondary star gravity acceleration [cgs]");
	declare_parameter ("phoebe_logg2_adjust",        PHOEBE, "component_logg2_adjust", "LOGG2ADJ", "");
	declare_parameter ("phoebe_logg2_step",          PHOEBE, "component_logg2_step_value", "LOGG2STEP", "");
	declare_parameter ("phoebe_logg2_min",           PHOEBE, "component_logg2_min_value", "LOGG2MIN", "");
	declare_parameter ("phoebe_logg2_max",           PHOEBE, "component_logg2_max_value", "LOGG2MAX", "");
	declare_parameter ("phoebe_met1_value",          PHOEBE, "component_met1_value", "MET1VAL", "Primary star metallicity [solar abundances]");
	declare_parameter ("phoebe_met1_adjust",         PHOEBE, "component_met1_adjust", "MET1ADJ", "");
	declare_parameter ("phoebe_met1_step",           PHOEBE, "component_met1_step_value", "MET1STEP", "");
	declare_parameter ("phoebe_met1_min",            PHOEBE, "component_met1_min_value", "MET1MIN", "");
	declare_parameter ("phoebe_met1_max",            PHOEBE, "component_met1_max_value", "MET1MAX", "");
	declare_parameter ("phoebe_met2_value",          PHOEBE, "component_met2_value", "MET2VAL", "Secondary star metallicity [solar abundances]");
	declare_parameter ("phoebe_met2_adjust",         PHOEBE, "component_met2_adjust", "MET2ADJ", "");
	declare_parameter ("phoebe_met2_step",           PHOEBE, "component_met2_step_value", "MET2STEP", "");
	declare_parameter ("phoebe_met2_min",            PHOEBE, "component_met2_min_value", "MET2MIN", "");
	declare_parameter ("phoebe_met2_max",            PHOEBE, "component_met2_max_value", "MET2MAX", "");

	/* *********************   Orbit tab parameters   ************************* */

	declare_parameter ("phoebe_ecc_value",           PHOEBE, "orbit_ecc_value", "ECCVAL", "Orbital eccentricity");
	declare_parameter ("phoebe_ecc_adjust",          PHOEBE, "orbit_ecc_adjust", "ECCADJ", "");
	declare_parameter ("phoebe_ecc_step",            PHOEBE, "orbit_ecc_step_value", "ECCSTEP", "");
	declare_parameter ("phoebe_ecc_min",             PHOEBE, "orbit_ecc_min_value", "ECCMIN", "");
	declare_parameter ("phoebe_ecc_max",             PHOEBE, "orbit_ecc_max_value", "ECCMAX", "");
	declare_parameter ("phoebe_perr0_value",         PHOEBE, "orbit_perr0_value", "PERR0VAL", "Argument of periastron");
	declare_parameter ("phoebe_perr0_adjust",        PHOEBE, "orbit_perr0_adjust", "PERR0ADJ", "");
	declare_parameter ("phoebe_perr0_step",          PHOEBE, "orbit_perr0_step_value", "PERR0STEP", "");
	declare_parameter ("phoebe_perr0_min",           PHOEBE, "orbit_perr0_min_value", "PERR0MIN", "");
	declare_parameter ("phoebe_perr0_max",           PHOEBE, "orbit_perr0_max_value", "PERR0MAX", "");
	declare_parameter ("phoebe_dperdt_value",        PHOEBE, "orbit_dperdt_value", "DPERDTVAL", "First time derivative of periastron");
	declare_parameter ("phoebe_dperdt_adjust",       PHOEBE, "orbit_dperdt_adjust", "DPERDTADJ", "");
	declare_parameter ("phoebe_dperdt_step",         PHOEBE, "orbit_dperdt_step_value", "DPERDTSTEP", "");
	declare_parameter ("phoebe_dperdt_min",          PHOEBE, "orbit_dperdt_min_value", "DPERDTMIN", "");
	declare_parameter ("phoebe_dperdt_max",          PHOEBE, "orbit_dperdt_max_value", "DPERDTMAX", "");
	declare_parameter ("phoebe_f1_value",            PHOEBE, "orbit_f1_value", "F1VAL", "Primary star synchronicity parameter");
	declare_parameter ("phoebe_f1_adjust",           PHOEBE, "orbit_f1_adjust", "F1ADJ", "");
	declare_parameter ("phoebe_f1_step",             PHOEBE, "orbit_f1_step_value", "F1STEP", "");
	declare_parameter ("phoebe_f1_min",              PHOEBE, "orbit_f1_min_value", "F1MIN", "");
	declare_parameter ("phoebe_f1_max",              PHOEBE, "orbit_f1_max_value", "F1MAX", "");
	declare_parameter ("phoebe_f2_value",            PHOEBE, "orbit_f2_value", "F2VAL", "Secondary star synchronicity parameter");
	declare_parameter ("phoebe_f2_adjust",           PHOEBE, "orbit_f2_adjust", "F2ADJ", "");
	declare_parameter ("phoebe_f2_step",             PHOEBE, "orbit_f2_step_value", "F2STEP", "");
	declare_parameter ("phoebe_f2_min",              PHOEBE, "orbit_f2_min_value", "F2MIN", "");
	declare_parameter ("phoebe_f2_max",              PHOEBE, "orbit_f2_max_value", "F2MAX", "");

	/* ********************   Surface tab parameters   ************************ */

	declare_parameter ("phoebe_alb1_value",          PHOEBE, "surface_alb1_value", "ALB1VAL", "Primary star albedo");
	declare_parameter ("phoebe_alb1_adjust",         PHOEBE, "surface_alb1_adjust", "ALB1ADJ", "");
	declare_parameter ("phoebe_alb1_step",           PHOEBE, "surface_alb1_step_value", "ALB1STEP", "");
	declare_parameter ("phoebe_alb1_min",            PHOEBE, "surface_alb1_min_value", "ALB1MIN", "");
	declare_parameter ("phoebe_alb1_max",            PHOEBE, "surface_alb1_max_value", "ALB1MAX", "");
	declare_parameter ("phoebe_alb2_value",          PHOEBE, "surface_alb2_value", "ALB2VAL", "Secondary star albedo");
	declare_parameter ("phoebe_alb2_adjust",         PHOEBE, "surface_alb2_adjust", "ALB2ADJ", "");
	declare_parameter ("phoebe_alb2_step",           PHOEBE, "surface_alb2_step_value", "ALB2STEP", "");
	declare_parameter ("phoebe_alb2_min",            PHOEBE, "surface_alb2_min_value", "ALB2MIN", "");
	declare_parameter ("phoebe_alb2_max",            PHOEBE, "surface_alb2_max_value", "ALB2MAX", "");
	declare_parameter ("phoebe_gr1_value",           PHOEBE, "surface_gr1_value", "GR1VAL", "Primary star gravity brightening");
	declare_parameter ("phoebe_gr1_adjust",          PHOEBE, "surface_gr1_adjust", "GR1ADJ", "");
	declare_parameter ("phoebe_gr1_step",            PHOEBE, "surface_gr1_step_value", "GR1STEP", "");
	declare_parameter ("phoebe_gr1_min",             PHOEBE, "surface_gr1_min_value", "GR1MIN", "");
	declare_parameter ("phoebe_gr1_max",             PHOEBE, "surface_gr1_max_value", "GR1MAX", "");
	declare_parameter ("phoebe_gr2_value",           PHOEBE, "surface_gr2_value", "GR2VAL", "Secondary star gravity brightening");
	declare_parameter ("phoebe_gr2_adjust",          PHOEBE, "surface_gr2_adjust", "GR2ADJ", "");
	declare_parameter ("phoebe_gr2_step",            PHOEBE, "surface_gr2_step_value", "GR2STEP", "");
	declare_parameter ("phoebe_gr2_min",             PHOEBE, "surface_gr2_min_value", "GR2MIN", "");
	declare_parameter ("phoebe_gr2_max",             PHOEBE, "surface_gr2_max_value", "GR2MAX", "");
	declare_parameter ("phoebe_hla_value",           PHOEBE, "flux_levelers_info_list_column_2", "HLAVAL", "");
	declare_parameter ("phoebe_hla_adjust",          PHOEBE, "surface_hla_adjust", "HLAADJ", "");
	declare_parameter ("phoebe_hla_step",            PHOEBE, "surface_hla_step_value", "HLASTEP", "");
	declare_parameter ("phoebe_cla_value",           PHOEBE, "flux_levelers_info_list_column_3", "CLAVAL", "");
	declare_parameter ("phoebe_cla_adjust",          PHOEBE, "surface_cla_adjust", "CLAADJ", "");
	declare_parameter ("phoebe_cla_step",            PHOEBE, "surface_cla_step_value", "CLASTEP", "");
	declare_parameter ("phoebe_decouplecla_switch",  PHOEBE, "surface_ipb_switch", "DECOUPLECLA", "Decouple secondary star surface brightness levels from temperature");
	declare_parameter ("phoebe_atm1_switch",         PHOEBE, "surface_ifat1_switch", "ATM1SWITCH", "Use stellar atmosphere model for primary star");
	declare_parameter ("phoebe_atm2_switch",         PHOEBE, "surface_ifat2_switch", "ATM2SWITCH", "Use stellar atmosphere model for secondary star");
	declare_parameter ("phoebe_el3_value",           PHOEBE, "third_light_info_list_column_2", "EL3VAL", "");
	declare_parameter ("phoebe_el3_adjust",          PHOEBE, "surface_el3_adjust", "EL3ADJ", "");
	declare_parameter ("phoebe_el3_step",            PHOEBE, "surface_el3_step_value", "EL3STEP", "");
	declare_parameter ("phoebe_opsf_value",          PHOEBE, "third_light_info_list_column_3", "OPSFVAL", "");
	declare_parameter ("phoebe_opsf_adjust",         PHOEBE, "surface_opsf_adjust", "OPSFADJ", "");
	declare_parameter ("phoebe_opsf_step",           PHOEBE, "surface_opsf_step_value", "OPSFSTEP", "");
	declare_parameter ("phoebe_reflection_switch",   PHOEBE, "surface_nref_switch", "NREFSWITCH", "Detailed reflection effect");
	declare_parameter ("phoebe_reflection_value",    PHOEBE, "surface_nref_value", "NREFVAL", "Number of reflections taken into account");
	declare_parameter ("phoebe_n1f_value",           PHOEBE, "surface_n1f_value",  "N1F",  "Fine grid size on primary star");
	declare_parameter ("phoebe_n2f_value",           PHOEBE, "surface_n2f_value",  "N2F",  "Fine grid size on secondary star");
	declare_parameter ("phoebe_n1c_value",           PHOEBE, "surface_n1c_value", "N1C", "Coarse grid size on primary star");
	declare_parameter ("phoebe_n2c_value",           PHOEBE, "surface_n2c_value", "N2C", "Coarse grid size on secondary star");

	/* ******************   Perturbations tab parameters   ******************** */

	declare_parameter ("phoebe_ldmodel_value",       PHOEBE, "ld_model_entry", "LDMODEL", "Limb darkening model");
	declare_parameter ("phoebe_xbol1_value",         PHOEBE, "ld_xbol1_value", "XBOL1",   "Limb darkening linear bolometric coefficient for primary star");
	declare_parameter ("phoebe_ybol1_value",         PHOEBE, "ld_ybol1_value", "YBOL1",   "Limb darkening non-linear bolometric coefficient for primary star");
	declare_parameter ("phoebe_xbol2_value",         PHOEBE, "ld_xbol2_value", "XBOL2",   "Limb darkening linear bolometric coefficient for secondary star");
	declare_parameter ("phoebe_ybol2_value",         PHOEBE, "ld_ybol2_value", "YBOL2",   "Limb darkening non-linear bolometric coefficient for secondary star");
	declare_parameter ("phoebe_lcx1_value",          PHOEBE, "ld_mono_lc_info_list_column_2", "LCX1VAL", "");
	declare_parameter ("phoebe_lcy1_value",          PHOEBE, "ld_mono_lc_info_list_column_3", "LCY1VAL", "");
	declare_parameter ("phoebe_lcx2_value",          PHOEBE, "ld_mono_lc_info_list_column_4", "LCX2VAL", "");
	declare_parameter ("phoebe_lcy2_value",          PHOEBE, "ld_mono_lc_info_list_column_5", "LCY2VAL", "");
	declare_parameter ("phoebe_lcx1_switch",         PHOEBE, "ld_x1_adjust", "LCX1ADJ", "");
	declare_parameter ("phoebe_lcx2_switch",         PHOEBE, "ld_x2_adjust", "LCX2ADJ", "");
	declare_parameter ("phoebe_lcx1_step",           PHOEBE, "ld_x1_step_value", "LCX1STEP", "");
	declare_parameter ("phoebe_lcx2_step",           PHOEBE, "ld_x2_step_value", "LCX2STEP", "");
	declare_parameter ("phoebe_rvx1_value",          PHOEBE, "ld_mono_rv_info_list_column_2", "RVX1VAL", "");
	declare_parameter ("phoebe_rvy1_value",          PHOEBE, "ld_mono_rv_info_list_column_3", "RVY1VAL", "");
	declare_parameter ("phoebe_rvx2_value",          PHOEBE, "ld_mono_rv_info_list_column_4", "RVX2VAL", "");
	declare_parameter ("phoebe_rvy2_value",          PHOEBE, "ld_mono_rv_info_list_column_5", "RVY2VAL", "");

	declare_parameter ("phoebe_sprim_value",         PHOEBE, "spots_sprim_value", "SPRIMVAL", "Number of spots on primary star");
	declare_parameter ("phoebe_ssec_value",          PHOEBE, "spots_ssec_value", "SSECVAL", "Number of spots on secondary star");
	declare_parameter ("phoebe_ifsmv1_switch",       PHOEBE, "spots_ifsmv1_switch", "IFSMV1", "Do the spots on primary star move?");
	declare_parameter ("phoebe_ifsmv2_switch",       PHOEBE, "spots_ifsmv2_switch", "IFSMV2", "Do the spots on secondary star move?");
	declare_parameter ("phoebe_lat1_value",          PHOEBE, "spots_primary_info_list_column_2", "LAT1VAL", "Latitude of the spot on primary star");
	declare_parameter ("phoebe_long1_value",         PHOEBE, "spots_primary_info_list_column_3", "LONG1VAL", "Longitude of the spot on primary star");
	declare_parameter ("phoebe_radsp1_value",        PHOEBE, "spots_primary_info_list_column_4", "RADSP1VAL", "Radius of the spot on primary star");
	declare_parameter ("phoebe_temsp1_value",        PHOEBE, "spots_primary_info_list_column_5", "TEMSP1VAL", "Temperature factor of the spot on primary star");
	declare_parameter ("phoebe_lat2_value",          PHOEBE, "spots_secondary_info_list_column_2", "LAT2VAL", "Latitude of the spot on secondary star");
	declare_parameter ("phoebe_long2_value",         PHOEBE, "spots_secondary_info_list_column_3", "LONG2VAL", "Longitude of the spot on secondary star");
	declare_parameter ("phoebe_radsp2_value",        PHOEBE, "spots_secondary_info_list_column_4", "RADSP2VAL", "Radius of the spot on secondary star");
	declare_parameter ("phoebe_temsp2_value",        PHOEBE, "spots_secondary_info_list_column_5", "TEMSP2VAL", "Temperature factor of the spot on secondary star");
	declare_parameter ("phoebe_lat1_adjust",         PHOEBE, "spots_xlat1_adjust", "LAT1ADJ", "");
	declare_parameter ("phoebe_long1_adjust",        PHOEBE, "spots_xlong1_adjust", "LONG1ADJ", "");
	declare_parameter ("phoebe_radsp1_adjust",       PHOEBE, "spots_radsp1_adjust", "RADSP1ADJ", "");
	declare_parameter ("phoebe_temsp1_adjust",       PHOEBE, "spots_temsp1_adjust", "TEMSP1ADJ", "");
	declare_parameter ("phoebe_lat2_adjust",         PHOEBE, "spots_xlat2_adjust", "LAT2ADJ", "");
	declare_parameter ("phoebe_long2_adjust",        PHOEBE, "spots_xlong2_adjust", "LONG2ADJ", "");
	declare_parameter ("phoebe_radsp2_adjust",       PHOEBE, "spots_radsp2_adjust", "RADSP2ADJ", "");
	declare_parameter ("phoebe_temsp2_adjust",       PHOEBE, "spots_temsp2_adjust", "TEMSP2ADJ", "");
	declare_parameter ("phoebe_lat1_step",           PHOEBE, "spots_xlat1_step_value", "LAT1STEP", "");
	declare_parameter ("phoebe_long1_step",          PHOEBE, "spots_xlong1_step_value", "LONG1STEP", "");
	declare_parameter ("phoebe_radsp1_step",         PHOEBE, "spots_radsp1_step_value", "RADSP1STEP", "");
	declare_parameter ("phoebe_temsp1_step",         PHOEBE, "spots_temsp1_step_value", "TEMSP1STEP", "");
	declare_parameter ("phoebe_lat2_step",           PHOEBE, "spots_xlat2_step_value", "LAT2STEP", "");
	declare_parameter ("phoebe_long2_step",          PHOEBE, "spots_xlong2_step_value", "LONG2STEP", "");
	declare_parameter ("phoebe_radsp2_step",         PHOEBE, "spots_radsp2_step_value", "RADSP2STEP", "");
	declare_parameter ("phoebe_temsp2_step",         PHOEBE, "spots_temsp2_step_value", "TEMSP2STEP", "");

	/* ********************   Utilities tab parameters   ********************** */

	declare_parameter ("phoebe_scatter_switch",      PHOEBE, "data_synthetic_scatter_switch", "SCATTER", "Synthetic scatter");
	declare_parameter ("phoebe_scatter_sigma",       PHOEBE, "data_synthetic_scatter_sigma_value", "SCATSIGMA", "Synthetic scatter standard deviation");
	declare_parameter ("phoebe_scatter_seed",        PHOEBE, "data_synthetic_scatter_seed_value", "SCATSEED", "Synthetic scatter seed");
	declare_parameter ("phoebe_scatter_weighting",   PHOEBE, "data_synthetic_scatter_weighting_list_entry", "SCATWEIGHTING", "Synthetic scatter weighting");
	declare_parameter ("phoebe_synthred_switch",     PHOEBE, "utilities_synthetic_reddening_switch", "SYNTHREDSWITCH", "Synthetic interstellar extinction (reddening)");
	declare_parameter ("phoebe_synthredr_value",     PHOEBE, "utilities_synthetic_reddening_r_value", "SYNTHREDFACTOR", "Synthetic interstellar extinction coefficient");
	declare_parameter ("phoebe_synthrede_value",     PHOEBE, "utilities_synthetic_reddening_e_value", "SYNTHREDEXCESS", "Synthetic interstellar extinction color excess value");

	/* ******************    Data statistics parameters     ******************* */

	declare_parameter ("phoebe_lc_ptsno_value",      PHOEBE_statistics, "statistics_lc_info_list_column_2", "LCPTSNO",   "Experimental data points number");
	declare_parameter ("phoebe_lc_sum_value",        PHOEBE_statistics, "statistics_lc_info_list_column_3", "LCSUM",     "Experimental data points sum");
	declare_parameter ("phoebe_lc_average_value",    PHOEBE_statistics, "statistics_lc_info_list_column_4", "LCAVERAGE", "Experimental data points average");
	declare_parameter ("phoebe_lc_median_value",     PHOEBE_statistics, "statistics_lc_info_list_column_5", "LCMEDIAN",  "Experimental data points median");
	declare_parameter ("phoebe_rv_ptsno_value",      PHOEBE_statistics, "statistics_rv_info_list_column_2", "RVPTSNO",   "Experimental data points number");
	declare_parameter ("phoebe_rv_sum_value",        PHOEBE_statistics, "statistics_rv_info_list_column_3", "RVSUM",     "Experimental data points sum");
	declare_parameter ("phoebe_rv_average_value",    PHOEBE_statistics, "statistics_rv_info_list_column_4", "RVAVERAGE", "Experimental data points average");
	declare_parameter ("phoebe_rv_median_value",     PHOEBE_statistics, "statistics_rv_info_list_column_5", "RVMEDIAN",  "Experimental data points median");

	/* ********************   LC plot window parameters   ********************* */

	declare_parameter ("phoebe_lc_indep_variable",   PHOEBE_plot_lc, "plot_lc_independent_variable_entry", "LCINDEP", "LC independent output variable");
	declare_parameter ("phoebe_lc_dep_variable",     PHOEBE_plot_lc, "plot_lc_dependent_variable_entry", "LCDEP", "LC dependent output variable");
	declare_parameter ("phoebe_lc_phase_start",      PHOEBE_plot_lc, "plot_lc_phase_start_value", "LCPHSTART",  "LC phase start");
	declare_parameter ("phoebe_lc_phase_end",        PHOEBE_plot_lc, "plot_lc_phase_end_value",   "LCPHEND",    "LC phase end");
	declare_parameter ("phoebe_lc_vertices_value",   PHOEBE_plot_lc, "plot_lc_vertices_value",    "LCVERTICES", "LC vertices number");
	declare_parameter ("phoebe_lc_alias_switch",     PHOEBE_plot_lc, "plot_lc_data_aliasing_switch", "LCALIAS", "LC Data aliasing");
	declare_parameter ("phoebe_lc_residuals_switch", PHOEBE_plot_lc, "plot_lc_plot_residuals_switch", "LCRESIDUALS", "LC residuals plot");
	declare_parameter ("phoebe_plot_lc_column_2",    PHOEBE_plot_lc, "plot_lc_info_list_column_2", "PLOTLCCOLUMN2", "");
	declare_parameter ("phoebe_plot_lc_column_3",    PHOEBE_plot_lc, "plot_lc_info_list_column_3", "PLOTLCCOLUMN3", "");
	declare_parameter ("phoebe_plot_lc_shift",       PHOEBE_plot_lc, "plot_lc_info_list_column_6", "PLOTLCSHIFT", "");
	declare_parameter ("phoebe_plot_lc_nexp_value",  PHOEBE_plot_lc, "plot_lc_statistics_info_list_column_2", "LCSTATNEXP",  "Experimental data points number");
	declare_parameter ("phoebe_plot_lc_aexp_value",  PHOEBE_plot_lc, "plot_lc_statistics_info_list_column_3", "LCSTATAEXP",  "Experimental data points average");
	declare_parameter ("phoebe_plot_lc_mexp_value",  PHOEBE_plot_lc, "plot_lc_statistics_info_list_column_4", "LCSTATMEXP",  "Experimental data points median");
	declare_parameter ("phoebe_plot_lc_asyn_value",  PHOEBE_plot_lc, "plot_lc_statistics_info_list_column_5", "LCSTATASYN",  "Synthetic data points average");
	declare_parameter ("phoebe_plot_lc_msyn_value",  PHOEBE_plot_lc, "plot_lc_statistics_info_list_column_6", "LCSTATMSYN",  "Synthetic data points median");
	declare_parameter ("phoebe_plot_lc_sigma_value", PHOEBE_plot_lc, "plot_lc_statistics_info_list_column_7", "LCSTATSIGMA", "Unweighted standard deviation");
	declare_parameter ("phoebe_plot_lc_wsig_value",  PHOEBE_plot_lc, "plot_lc_statistics_info_list_column_8", "LCSTATWSIG",  "Weighted standard deviation");
	}

GtkWidget *phoebe_widget_from_qualifier (char *qualifier)
	{
	/* This function returns the parent widget of the given qualifier.          */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( strcmp (PHOEBE_parameter.qualifier[i], qualifier) == 0 )
			return PHOEBE_parameter.widget[i];

	printf ("*** Warning: phoebe_widget_from_qualifier () called with qualifier\n'%s', which doesn't exist!\n", qualifier);
	return NULL;
	}

GtkWidget *phoebe_widget_from_keyword (char *keyword)
	{
	/* This function returns the parent widget of the given keyword.            */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( strcmp (PHOEBE_parameter.keyword[i], keyword) == 0 )
			return PHOEBE_parameter.widget[i];

	printf ("*** Warning: phoebe_widget_from_keyword () called with keyword\n'%s', which doesn't exist!\n", keyword);
	return NULL;
	}

GtkWidget *phoebe_widget_from_description (char *description)
	{
	/* This function returns the parent widget of the given description.        */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( strcmp (PHOEBE_parameter.description[i], description) == 0 )
			return PHOEBE_parameter.widget[i];

	printf ("*** Warning: phoebe_widget_from_description () called with description\n'%s', which doesn't exist!\n", description);
	return NULL;
	}

char *phoebe_qualifier_from_widget (GtkWidget *widget)
	{
	/* This function returns the qualifier of the parent widget.                */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( widget == PHOEBE_parameter.widget[i] )
			return PHOEBE_parameter.qualifier[i];

	printf ("*** Warning: phoebe_qualifier_from_widget () called with non-existing widget\n");
	return NULL;
	}

char *phoebe_qualifier_from_keyword (char *keyword)
	{
	/* This function returns the qualifier of the given keyword.                */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( strcmp (PHOEBE_parameter.keyword[i], keyword) == 0 )
			return PHOEBE_parameter.qualifier[i];

	printf ("*** Warning: phoebe_qualifier_from_keyword () called with keyword\n'%s' which doesn't exist!\n", keyword);
	return NULL;
	}

char *phoebe_qualifier_from_description (char *description)
	{
	/* This function returns the qualifier of the given description.            */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( strcmp (PHOEBE_parameter.description[i], description) == 0 ) return PHOEBE_parameter.qualifier[i];

	printf ("*** Warning: phoebe_qualifier_from_description () called with description\n'%s', which doesn't exist!\n", description);
	return NULL;
	}

char *phoebe_keyword_from_widget (GtkWidget *widget)
	{
	/* This function returns the keyword of the parent widget.                  */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( widget == PHOEBE_parameter.widget[i] ) return PHOEBE_parameter.keyword[i];

	printf ("*** Warning: phoebe_keyword_from_widget () called with non-existing widget\n");
	return NULL;
	}

char *phoebe_keyword_from_qualifier (char *qualifier)
	{
	/* This function returns the keyword of the given qualifier.                */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( strcmp (PHOEBE_parameter.qualifier[i], qualifier) == 0 ) return PHOEBE_parameter.keyword[i];

	printf ("*** Warning: phoebe_keyword_from_qualifier () called with qualifier\n'%s', which doesn't exist!\n", qualifier);
	return NULL;
	}

char *phoebe_keyword_from_description (char *description)
	{
	/* This function returns the keyword of the given description.              */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( strcmp (PHOEBE_parameter.description[i], description) == 0 ) return PHOEBE_parameter.keyword[i];

	printf ("*** Warning: phoebe_keyword_from_description () called with description\n'%s', which doesn't exist!\n", description);
	return NULL;
	}

char *phoebe_description_from_widget (GtkWidget *widget)
	{
	/* This function returns the description of the parent widget.              */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( widget == PHOEBE_parameter.widget[i] ) return PHOEBE_parameter.description[i];

	printf ("*** Warning: phoebe_description_from_widget () called with non-existing widget\n");
	return NULL;
	}

char *phoebe_description_from_qualifier (char *qualifier)
	{
	/* This function returns the description of the given qualifier.            */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( strcmp (PHOEBE_parameter.qualifier[i], qualifier) == 0 ) return PHOEBE_parameter.description[i];

	printf ("*** Warning: phoebe_description_from_qualifier () called with qualifier\n'%s', which doesn't exist!\n", qualifier);
	return NULL;
	}

char *phoebe_description_from_keyword (char *keyword)
	{
	/* This function returns the description of the given keyword.              */
	
	int i;
	
	for (i = 0; i < PHOEBE_parameter.recno; i++)
		if ( strcmp (PHOEBE_parameter.keyword[i], keyword) == 0 ) return PHOEBE_parameter.description[i];

	printf ("*** Warning: phoebe_description_from_keyword () called with keyword\n'%s', which doesn't exist!\n", keyword);
	return NULL;
	}

int phoebe_get_value_int (char *qualifier)
	{
	/* This function returns the integer value behind the given delimeter.      */

	return gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (phoebe_widget_from_qualifier (qualifier)));
	}

double phoebe_get_value_double (char *qualifier)
	{
	/* This function returns the double value behind the given delimeter.       */

	return gtk_spin_button_get_value (GTK_SPIN_BUTTON (phoebe_widget_from_qualifier (qualifier)));
	}

bool phoebe_get_value_bool (char *qualifier)
	{
	/* This function returns the boolean value behind the given delimeter.      */

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (phoebe_widget_from_qualifier (qualifier))) == TRUE)
		return YES;
	else return NO;
	}

const char *phoebe_get_value_string (char *qualifier)
	{
	/* This function returns the string value behind the given delimeter.       */

	return gtk_entry_get_text (GTK_ENTRY (phoebe_widget_from_qualifier (qualifier)));
	}

char *phoebe_get_value_list_string (char *qualifier, int row)
	{
	/* This function returns the contents of the GtkTreeView widget; the column */
	/* is defined by the qualifier and the row by the "row" argument.           */

	GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN (phoebe_widget_from_qualifier (qualifier));
	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (column->tree_view));
	GtkTreeIter iter;

	char working_string[3];
	char *working_str = working_string;
	char *readout_str;

	sprintf (working_str, "%d", row);
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (model), &iter, working_str);
	gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, gtk_tree_view_column_get_index (column), &readout_str, -1);

	return readout_str;
	}

double phoebe_get_value_list_double (char *qualifier, int row)
	{
	/* This function returns the contents of the GtkTreeView widget; the column */
	/* is defined by the qualifier and the row by the "row" argument.           */

	GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN (phoebe_widget_from_qualifier (qualifier));
	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (column->tree_view));
	GtkTreeIter iter;

	char working_string[3];
	char *working_str = working_string;

	double return_val;

	sprintf (working_str, "%d", row);
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (model), &iter, working_str);
	gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, gtk_tree_view_column_get_index (column), &return_val, -1);

	return return_val;
	}

bool phoebe_get_value_list_bool (char *qualifier, int row)
	{
	GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN (phoebe_widget_from_qualifier (qualifier));
	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (column->tree_view));
	GtkTreeIter iter;

	char working_string[3];
	char *working_str = working_string;

	gboolean state;

	sprintf (working_str, "%d", row);
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (model), &iter, working_str);
	gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, gtk_tree_view_column_get_index (column), &state, -1);

	return state;
	}

void phoebe_set_value_int (char *qualifier, int value)
	{
	/* This function writes the integer value to the given qualifier.           */

	gtk_spin_button_set_value (GTK_SPIN_BUTTON (phoebe_widget_from_qualifier (qualifier)), value);
	}

void phoebe_set_value_double (char *qualifier, double value)
	{
	/* This function writes the double value to the given qualifier.            */

	gtk_spin_button_set_value (GTK_SPIN_BUTTON (phoebe_widget_from_qualifier (qualifier)), value);
	}

void phoebe_set_value_bool (char *qualifier, bool value)
	{
	/* This function writes the boolean value to the given qualifier.           */

	if (value == YES) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (phoebe_widget_from_qualifier (qualifier)), TRUE);
		else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (phoebe_widget_from_qualifier (qualifier)), FALSE);
	}

void phoebe_set_value_string (char *qualifier, const char *value)
	{
	/* This function writes the string value to the given qualifier.            */

	gtk_entry_set_text (GTK_ENTRY (phoebe_widget_from_qualifier (qualifier)), value);
	}

void phoebe_set_value_list_string (char *qualifier, int row, const char *value)
	{
	/* This function writes the string to the column and row of the Tree View,  */
	/* which is represented by qualifier and row.                               */

	GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN (phoebe_widget_from_qualifier (qualifier));
	GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (column->tree_view)));
	GtkTreeIter iter;

	char row_string[5];
	char *row_str = row_string;

	sprintf (row_str, "%d", row);
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, row_str);
	gtk_list_store_set (store, &iter, gtk_tree_view_column_get_index (column), value, -1);
	}

void phoebe_set_value_list_int (char *qualifier, int row, int value)
	{
	/* This function writes the string to the column and row of the Tree View,  */
	/* which is represented by qualifier and row.                               */

	GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN (phoebe_widget_from_qualifier (qualifier));
	GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (column->tree_view)));
	GtkTreeIter iter;

	char row_string[5];
	char *row_str = row_string;

	sprintf (row_str, "%d", row);
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, row_str);
	gtk_list_store_set (store, &iter, gtk_tree_view_column_get_index (column), value, -1);
	}

void phoebe_set_value_list_double (char *qualifier, int row, double value)
	{
	/* This function writes the string to the column and row of the Tree View,  */
	/* which is represented by qualifier and row.                               */

	GtkTreeViewColumn *column = GTK_TREE_VIEW_COLUMN (phoebe_widget_from_qualifier (qualifier));
	GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (column->tree_view)));
	GtkTreeIter iter;

	char row_string[5];
	char *row_str = row_string;

	sprintf (row_str, "%d", row);
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, row_str);
	gtk_list_store_set (store, &iter, gtk_tree_view_column_get_index (column), value, -1);
	}

PHOEBE_input_indep get_input_independent_variable (char *value)
	{
	if (strcmp (value, "Phase")      == 0) return INPUT_PHASE;
	if (strcmp (value, "Time (HJD)") == 0) return INPUT_HJD;
	}

PHOEBE_input_dep get_input_dependent_variable (char *value)
	{
	if (strcmp (value, "Magnitude")  == 0) return INPUT_MAGNITUDE;
	if (strcmp (value, "Flux")       == 0) return INPUT_FLUX;
	if (strcmp (value, "RV in km/s") == 0) return INPUT_RV_IN_KMS;
	}

PHOEBE_input_weight get_input_weight (char *value)
	{
	if (strcmp (value, "Standard weight (integer)")  == 0) return INPUT_STANDARD_WEIGHT;
	if (strcmp (value, "Standard weight (double)")   == 0) return INPUT_STANDARD_WEIGHT;
	if (strcmp (value, "Standard deviation (sigma)") == 0) return INPUT_STANDARD_DEVIATION;
	if (strcmp (value, "Unavailable")                == 0) return INPUT_UNAVAILABLE;
	}

PHOEBE_output_indep get_output_independent_variable (char *value)
	{
	if (strcmp (value, "Phase")      == 0) return OUTPUT_PHASE;
	if (strcmp (value, "Time (HJD)") == 0) return OUTPUT_HJD;
	}

PHOEBE_output_dep get_output_dependent_variable (char *value)
	{
	if (strcmp (value, "Magnitude")               == 0) return OUTPUT_MAGNITUDE;
	if (strcmp (value, "Total flux")              == 0) return OUTPUT_TOTAL_FLUX;
	if (strcmp (value, "Primary star flux")       == 0) return OUTPUT_PRIMARY_FLUX;
	if (strcmp (value, "Secondary star flux")     == 0) return OUTPUT_SECONDARY_FLUX;
	if (strcmp (value, "Primary RV")              == 0) return OUTPUT_PRIMARY_RV;
	if (strcmp (value, "Secondary RV")            == 0) return OUTPUT_SECONDARY_RV;
	if (strcmp (value, "Both RVs")                == 0) return OUTPUT_BOTH_RVS;
	if (strcmp (value, "Primary normalized RV")   == 0) return OUTPUT_PRIMARY_NORMALIZED_RV;
	if (strcmp (value, "Secondary normalized RV") == 0) return OUTPUT_SECONDARY_NORMALIZED_RV;
	}

PHOEBE_output_weight get_output_weight (char *value)
	{
	if (strcmp (value, "Standard weight")            == 0) return OUTPUT_STANDARD_WEIGHT;
	if (strcmp (value, "Standard deviation (sigma)") == 0) return OUTPUT_STANDARD_DEVIATION;
	if (strcmp (value, "Unavailable")                == 0) return OUTPUT_UNAVAILABLE;
	}

void get_from_keyword_file (char *qualifier, char *value_str)
	{
	/* This function reads the PHOEBE qualifier, assigns a widget to it and     */
	/* fills the read value from the file to it.                                */

	GtkWidget *widget = phoebe_widget_from_qualifier (qualifier);

	if (GTK_IS_SPIN_BUTTON (widget))
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), atof (value_str));
	if ( (GTK_IS_ENTRY (widget)) && (!GTK_IS_SPIN_BUTTON (widget)) )
		{
		/* Strings are "dressed" in double quotes, we have to get rid of them:    */
		while (value_str[0] == '"') value_str++;
		while (value_str[strlen(value_str)-1] == '"') strip_string_tail (value_str);
		gtk_entry_set_text (GTK_ENTRY (widget), value_str);
		}
	if (GTK_IS_TOGGLE_BUTTON (widget))
		if (strcmp (value_str, "0") == 0) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
			else gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
	}

void get_wavelength_dependent_parameter_from_keyword_file (char *qualifier, char *index, char *value_str)
	{
	/* Much like the function above, only that this one takes three arguments,  */
	/* the 2nd one being the index of the wavelength-dependent parameter.       */

	GtkTreeViewColumn *column;
	GtkTreeModel *model;
	GtkTreeIter iter;

	int i = atof (index) - 1;
	char index_string[3];
	char *ind = index_string;

	/* First, let's check whether the qualifier is indeed a wavelength-depen-   */
	/* dent parameter; if it is, it is stored in a TreeView column.             */
	if (!GTK_IS_TREE_VIEW_COLUMN (phoebe_widget_from_qualifier (qualifier)))
		{
		phoebe_warning ("Parameter %s is invalid! Please review PHOEBE 0.30 keyword specifications!\n", qualifier);
		return;
		}

	column = GTK_TREE_VIEW_COLUMN (phoebe_widget_from_qualifier (qualifier));
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (column->tree_view));

	/* Strings are "dressed" in double quotes, we have to get rid of them:    */
	if (value_str[0] == '"') value_str++;
	if (value_str[strlen(value_str)-1] == '"') strip_string_tail (value_str);
	
	sprintf (ind, "%d", i);
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (model), &iter, ind);

	if (gtk_tree_model_get_column_type (model, gtk_tree_view_column_get_index (column)) == G_TYPE_STRING)
		gtk_list_store_set (GTK_LIST_STORE (model), &iter, gtk_tree_view_column_get_index (column), value_str, -1);
	if (gtk_tree_model_get_column_type (model, gtk_tree_view_column_get_index (column)) == G_TYPE_DOUBLE)
		gtk_list_store_set (GTK_LIST_STORE (model), &iter, gtk_tree_view_column_get_index (column), atof (value_str), -1);
	}

int open_keyword_file (const char *filename)
	{
	char readout_string[256];
	char *readout_str = readout_string;

	char keyword_string[256];
	char *keyword_str = keyword_string;

	char value_string[256];
	char *value_str = value_string;

	char working_string[256];
	char *working_str = working_string;

	int i;

	FILE *keyword_file = fopen (filename, "r");

	while (feof (keyword_file) == 0)
		{
		fgets (readout_str, 255, keyword_file);
		if (feof (keyword_file) != 0) break;

		/* fgets reads a newline character, which we don't want really:           */
		strip_string_tail (readout_str);

		/* Let us check if the keyword file is an old PHOEBE keyword file:        */
		if (strchr (readout_str, '=') == NULL)
			{
			fclose (keyword_file);
			open_legacy_keyword_file (filename);
			return 1;
			}

		sscanf (readout_str, "%s = %*s", keyword_str);
		value_str = strchr (readout_str, '=');
		if (value_str == NULL)
			{
			/* If the keyword doesn't have '=', it will be skipped.                 */
			phoebe_warning ("keyword %s in file %s invalid.", keyword_str, filename);
			continue;
			}

		/* value_str now points to '=', we need the next character; beware that   */
		/* this increment offsets the pointer within the char array, thus we have */
		/* to repoint at the end of the loop.                                     */
		value_str++;

		/* Eat all empty spaces at the beginning and at the end:                  */
		while (value_str[0] == ' ') value_str++;
		while (value_str[strlen (value_str)-1] == ' ') strip_string_tail (value_str);

		for (i = 0; i < PHOEBE_parameter.recno; i++)
			{
			if (strcmp (keyword_str, PHOEBE_parameter.keyword[i]) == 0)
				{
				get_from_keyword_file (PHOEBE_parameter.qualifier[i], value_str);
				break;
				}
			if (strncmp (keyword_str, PHOEBE_parameter.keyword[i], strlen (PHOEBE_parameter.keyword[i])) == 0)
				{
				/* This is appopriate for wavelength-dependent parameters; a consecu- */
				/* tive number is attached to the end of the keyword for distinction. */

				keyword_str += strlen (PHOEBE_parameter.keyword[i]);
				get_wavelength_dependent_parameter_from_keyword_file (PHOEBE_parameter.qualifier[i], keyword_str, value_str);
				break;
				}
			if (i == PHOEBE_parameter.recno - 1)
				printf ("The keyword '%s' isn't recognized by PHOEBE. Sorry.\n", keyword_str);
			}

		/* As explained above, we have to repoint strings:                        */
		readout_str = readout_string;
		keyword_str = keyword_string;
		value_str = value_string;
		working_str = working_string;
		}

	fclose (keyword_file);

	return 0;
	}

void save_to_keyword_file (char *param, FILE *file)
	{
	GtkWidget *widget = phoebe_widget_from_keyword (param);
	int i, rows;
	GtkTreeModel *model;

	if ( GTK_IS_SPIN_BUTTON (widget) )
		{
		if (gtk_spin_button_get_digits (GTK_SPIN_BUTTON (widget)) == 0)
			fprintf (file, "%s = %d\n", param, gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (widget)));
		else
			fprintf (file, "%s = %lf\n", param, gtk_spin_button_get_value (GTK_SPIN_BUTTON (widget)));
		}
	if ( GTK_IS_ENTRY (widget) && !GTK_IS_SPIN_BUTTON (widget) )
		fprintf (file, "%s = \"%s\"\n", param, gtk_entry_get_text (GTK_ENTRY (widget)));
	if ( GTK_IS_TOGGLE_BUTTON (widget) )
		{
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) == TRUE)
			fprintf (file, "%s = %d\n", param, 1);
		else
			fprintf (file, "%s = %d\n", param, 0);
		}
	if ( GTK_IS_TREE_VIEW_COLUMN (widget) )
		{
		model = gtk_tree_view_get_model (GTK_TREE_VIEW (GTK_TREE_VIEW_COLUMN (widget)->tree_view));
		rows = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), NULL);
		for (i = 0; i < rows; i++)
			{
			if (gtk_tree_model_get_column_type (model, gtk_tree_view_column_get_index (GTK_TREE_VIEW_COLUMN (widget))) == G_TYPE_STRING)
				fprintf (file, "%s%d = \"%s\"\n", param, i+1, phoebe_get_value_list_string (phoebe_qualifier_from_widget (widget), i));
			if (gtk_tree_model_get_column_type (model, gtk_tree_view_column_get_index (GTK_TREE_VIEW_COLUMN (widget))) == G_TYPE_DOUBLE)
				fprintf (file, "%s%d = %lf\n", param, i+1, phoebe_get_value_list_double (phoebe_qualifier_from_widget (widget), i));
			}
		}
	}

void save_keyword_file (const char *filename)
	{
	FILE *keyword_file = fopen (filename, "w");
	int i;

	for (i = 0; i < PHOEBE_parameter.recno; i++)
		save_to_keyword_file (PHOEBE_parameter.keyword[i], keyword_file);

	fclose (keyword_file);
	}

void open_legacy_keyword_file (const char *filename)
	{
	phoebe_warning ("Not yet implemented!\n");
	}
