#ifndef PHOEBE_KEYWORDS_H
	#define PHOEBE_KEYWORDS_H 1

#include "phoebe_global.h"

/* Functions for manipulating with internal PHOEBE keywords:                  */

void declare_parameter (char *qualifier, GtkWidget *parent, char *widget, char *keyword, char *description);
void declare_all_parameters ();

GtkWidget *phoebe_widget_from_qualifier   (char *qualifier);
GtkWidget *phoebe_widget_from_keyword     (char *keyword);
GtkWidget *phoebe_widget_from_description (char *description);
char *phoebe_qualifier_from_widget        (GtkWidget *widget);
char *phoebe_qualifier_from_keyword       (char *keyword);
char *phoebe_qualifier_from_description   (char *description);
char *phoebe_keyword_from_widget          (GtkWidget *widget);
char *phoebe_keyword_from_qualifier       (char *qualifier);
char *phoebe_keyword_from_description     (char *description);
char *phoebe_description_from_widget      (GtkWidget *widget);
char *phoebe_description_from_qualifier   (char *qualifier);
char *phoebe_description_from_keyword     (char *description);

int         phoebe_get_value_int         (char *qualifier);
double      phoebe_get_value_double      (char *qualifier);
bool        phoebe_get_value_bool        (char *qualifier);
const char *phoebe_get_value_string      (char *qualifier);
char       *phoebe_get_value_list_string (char *qualifier, int row);
double      phoebe_get_value_list_double (char *qualifier, int row);
bool        phoebe_get_value_list_bool   (char *qualifier, int row);

PHOEBE_input_indep   get_input_independent_variable  (char *value);
PHOEBE_input_dep     get_input_dependent_variable    (char *value);
PHOEBE_input_weight  get_input_weight                (char *value);
PHOEBE_output_indep  get_output_independent_variable (char *value);
PHOEBE_output_dep    get_output_dependent_variable   (char *value);
PHOEBE_output_weight get_output_weight               (char *value);

void        phoebe_set_value_int         (char *qualifier, int value);
void        phoebe_set_value_double      (char *qualifier, double value);
void        phoebe_set_value_bool        (char *qualifier, bool value);
void        phoebe_set_value_string      (char *qualifier, const char *value);
void        phoebe_set_value_list_string (char *qualifier, int row, const char *value);
void        phoebe_set_value_list_int    (char *qualifier, int row, int value);
void        phoebe_set_value_list_double (char *qualifier, int row, double value);

/* Opening and saving keyword files:                                          */

int open_keyword_file (const char *filename);

void get_from_keyword_file (char *qualifier, char *value_str);
void save_to_keyword_file (char *param, FILE *file);
void save_keyword_file (const char *filename);
void open_legacy_keyword_file (const char *filename);

#endif
