#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "phoebe_global.h"
#include "phoebe_scripting.h"

int declare_script_command (char *name, char *description, int (*func)(), int argc, ...)
	{
	/* This function takes variable number of arguments: first 4 are fixed (na- */
	/* me, description, function and the number of arguments) and the following */
	/* depend on argc and are all of PHOEBE_type that define the type of the    */
	/* arguments passed to the script function.                                 */

	va_list args;

	int i = PHOEBE_scripter_commands_no;
	int j;

	PHOEBE_scripter_commands_no++;
	PHOEBE_scripter_commands = realloc (PHOEBE_scripter_commands, PHOEBE_scripter_commands_no * sizeof (*PHOEBE_scripter_commands));

	PHOEBE_scripter_commands[i].name        = strdup (name);
	PHOEBE_scripter_commands[i].description = strdup (description);
	PHOEBE_scripter_commands[i].argc        = argc;
	PHOEBE_scripter_commands[i].function    = func;
	PHOEBE_scripter_commands[i].types       = malloc (argc * sizeof (PHOEBE_type));
	PHOEBE_scripter_commands[i].argnames    = malloc (argc * sizeof (char *));
	va_start (args, argc);
	for (j = 0; j < 2*argc; j+=2)
		{
		PHOEBE_scripter_commands[i].types[j/2]     = va_arg (args, int);
		PHOEBE_scripter_commands[i].argnames[j/2]  = va_arg (args, char *);
		}
	va_end (args);

	return i+1;
	}

void declare_all_scripter_commands ()
	{
	declare_script_command ("open_keyword_file", "Open PHOEBE Keyword File", script_open_keyword_file, 1, TYPE_STRING, "Filename");
	declare_script_command ("save_keyword_file", "Save PHOEBE Keyword File", script_dummy,             1, TYPE_STRING, "Filename");
	}

int execute_script_command (char *name, ...)
	{
	/* This is generic script execution function. It reads the name from the    */
	/* argument, cross-identifies it with the global PHOEBE_scripter_command    */
	/* table and gets all relevant info from that table.                        */

	int i = 0;
	int argc;
	int (*func)(va_list *);

	char *readout_str;

	va_list args;

	while ( (i < PHOEBE_scripter_commands_no) && (strcmp (PHOEBE_scripter_commands[i].name, name) != 0) ) i++;

	if (strcmp (PHOEBE_scripter_commands[i].name, name) != 0)
		{
		phoebe_warning ("Command %s not found, aborting execution.", name);
		return -1;
		}

	/* Get the number of arguments to the specified function:                   */
	argc = PHOEBE_scripter_commands[i].argc;

	/* Get the execution function:                                              */
	func = PHOEBE_scripter_commands[i].function;

	va_start (args, name);
	func (&args);
	va_end (args);
	}

int script_open_keyword_file (va_list *args)
	{
	char *readout_str;

	readout_str = va_arg (*args, char *);
	open_keyword_file (readout_str);
	}

int script_dummy (va_list *args)
	{
	phoebe_notice ("This script command is not yet implemented.\n");
	}
